var currencyFormatter = function (value) {
    if (typeof value !== 'number')
        return NaN;
    return new Intl.NumberFormat('pt-BR', {
        style: 'currency',
        currency: 'BRL',
    }).format(value);
};

var maskCleaner = function (barcode) {
    var onlyNumbers = barcode.match(/\d+/g);
    return !onlyNumbers ? '' : onlyNumbers.join('');
};

var typeMapping = function (option) {
    var config = [
        { type: 'ARRECADACAO', subtype: '' },
        { type: 'ARRECADACAO', subtype: 'ARRECADACAO_PREFEITURA' },
        { type: 'ARRECADACAO', subtype: 'CONVENIO_SANEAMENTO' },
        { type: 'ARRECADACAO', subtype: 'CONVENIO_ENERGIA_ELETRICA_E_GAS' },
        { type: 'ARRECADACAO', subtype: 'CONVENIO_TELECOMUNICACOES' },
        { type: 'ARRECADACAO', subtype: 'ARRECADACAO_ORGAOS_GOVERNAMENTAIS' },
        { type: 'ARRECADACAO', subtype: 'OUTROS' },
        { type: 'ARRECADACAO', subtype: 'ARRECADACAO_TAXAS_DE_TRANSITO' },
        { type: 'ARRECADACAO', subtype: '' },
        { type: 'ARRECADACAO', subtype: 'OUTROS' },
        { type: 'ARRECADACAO', subtype: '' },
    ];
    return config[option % 10];
};

var differenceForNow = function (date) {
    var dRefNow = new Date();
    var now = new Date(dRefNow.getMonth() + 1 + "/" + dRefNow.getDate() + "/" + dRefNow.getFullYear());
    var past = new Date(date.getMonth() + 1 + "/" + date.getDate() + "/" + date.getFullYear());
    var diff = Math.abs(now.getTime() - past.getTime());
    var days = Math.ceil(diff / (1000 * 60 * 60 * 24));
    var isPast = now.getTime() > past.getTime();
    return [days, isPast];
};

var identifyBank = function (cod) {
    var banks = {
        33: 'Banco Santander (Brasil) S.A',
        36: 'Banco Bradesco BBI S.A',
        341: 'Itaú Unibanco S.A',
        104: 'Caixa Econômica Federal',
        1: 'Banco do Brasil',
        7: 'BNDES',
        69: 'Crefisa',
        77: 'Banco Inter',
        102: 'XP Investimentos',
        140: 'Easynvest',
        197: 'Stone',
        208: 'BTG Pactual',
        212: 'Banco Original',
        237: 'Bradesco',
        260: 'Nu Pagamentos',
        422: 'Banco Safra',
        633: 'Banco Rendimento',
        652: 'Itaú Unibanco',
        735: 'Banco Neon',
        739: 'Banco Cetelem',
        745: 'Citibank',
    };
    return banks[Number(cod)] || 'Unknown';
};

var interstCalc = function (valueBoleto, daysExpired, feesValue, type, percent, month) {
    if (type === 'BANCO') {
        if (!feesValue && typeof feesValue !== 'number') {
            throw new TypeError('Tipo de valor informado para juros é inválido.');
        }
        if (feesValue && percent && month) {
            return (((feesValue / 100) / 30) * daysExpired) * valueBoleto;
        }
        if (feesValue && percent && !month) {
            return ((feesValue / 100) * daysExpired) * valueBoleto;
        }
        if (feesValue && !percent && month) {
            return ((feesValue / 30) * daysExpired);
        }
        if (feesValue && !percent && !month) {
            return (feesValue * daysExpired);
        }
    }
    return 0;
};
var finesCalc = function (valueBoleto, expired, type, finesValue, percent) {
    if (type === 'BANCO' && expired) {
        if (!finesValue && typeof finesValue !== 'number') {
            throw new TypeError('Tipo de valor informado para multa é inválido.');
        }
        if (finesValue && percent)
            return (finesValue / 100) * valueBoleto;
        if (finesValue && !percent)
            return finesValue;
    }
    return 0;
};

var module10 = function (block) {
    var summation = block.split('').reverse().reduce(function (acc, current, i) {
        var sum = Number(current) * (((i + 1) % 2) + 1);
        sum = (sum > 9 ? Math.trunc(sum / 10) + (sum % 10) : sum);
        return acc + sum;
    }, 0);
    return (Math.ceil(summation / 10) * 10) - summation;
};
var module11ForBankType = function (block) {
    var factor = 2;
    var summation = block.split('').reverse().reduce(function (acc, current) {
        var sum = Number(current) * factor;
        factor = factor === 9 ? 2 : factor + 1;
        return acc + sum;
    }, 0);
    var rest = summation % 11;
    var DV = 11 - rest;
    if (DV === 0 || DV === 10 || DV === 11)
        return 1;
    return DV;
};
var module11ForAgreementType = function (block) {
    var factor = 2;
    var summation = block.split('').reverse().reduce(function (acc, current) {
        var sum = Number(current) * factor;
        factor = factor === 9 ? 2 : factor + 1;
        return acc + sum;
    }, 0);
    var rest = summation % 11;
    if (rest === 0 || rest === 1 || rest === 10) {
        return rest === 0 || rest === 1 ? 0 : 1;
    }
    return 11 - rest;
};

var toBarcodeBank = function (number) { return number.replace(/^(\d{4})(\d{5})\d{1}(\d{10})\d{1}(\d{10})\d{1}(\d{15})$/, '$1$5$2$3$4'); };
var toBarcodeAgreement = function (number) {
    var barcode = '';
    for (var index = 0; index < 4; index += 1) {
        var start = (11 * (index)) + index;
        var end = (11 * (index + 1)) + index;
        barcode += number.substring(start, end);
    }
    return barcode;
};

var barcodeBank = function (number) {
    if (number.length !== 44)
        return false;
    var DV = number.substr(4, 1);
    var block = number.substr(0, 4) + number.substr(5);
    return module11ForBankType(block) === Number(DV);
};
var barcodeAgreement = function (number) {
    if (number.length !== 44 || Number(number.substr(0, 1)) !== 8)
        return false;
    var coin = Number(number.substr(2, 1));
    var DV = Number(number.substr(3, 1));
    var block = number.substr(0, 3) + number.substr(4);
    var moduleRef = null;
    if (coin === 6 || coin === 7)
        moduleRef = module10;
    if (coin === 8 || coin === 9)
        moduleRef = module11ForAgreementType;
    if (!moduleRef)
        return false;
    return moduleRef(block) === DV;
};
var digitableLineBank = function (number, blockValidate) {
    if (blockValidate === void 0) { blockValidate = false; }
    if (number.length !== 47)
        return false;
    var blocks = [
        {
            num: number.substr(0, 9),
            DV: number.substr(9, 1),
        },
        {
            num: number.substr(10, 20),
            DV: number.substr(20, 1),
        },
        {
            num: number.substr(21, 31),
            DV: number.substr(31, 1),
        },
    ];
    var blocksValidate = blockValidate
        ? blocks.every(function (e) { return module10(e.num) === Number(e.DV); })
        : true;
    var checksum = barcodeBank(toBarcodeBank(number));
    return blocksValidate && checksum;
};
var digitableLineAgreement = function (number, blockValidate) {
    if (blockValidate === void 0) { blockValidate = false; }
    if (number.length !== 48 || Number(number.substr(0, 1)) !== 8)
        return false;
    var validDV = barcodeAgreement(toBarcodeAgreement(number));
    if (!blockValidate)
        return validDV;
    var coin = Number(number.substr(2, 1));
    var moduleRef = null;
    if (coin === 6 || coin === 7)
        moduleRef = module10;
    if (coin === 8 || coin === 9)
        moduleRef = module11ForAgreementType;
    if (!moduleRef)
        return false;
    var blocks = Array.from({ length: 4 }, function (v, index) {
        var start = (11 * (index)) + index;
        var end = (11 * (index + 1)) + index;
        return {
            num: number.substr(start, end),
            DV: number.substr(end, end + 1),
        };
    });
    var validblocks = blocks.every(function (e) { return moduleRef && moduleRef(e.num) === Number(e.DV); });
    return validblocks && validDV;
};

var isValid = function (number, blocksValidate) {
    if (blocksValidate === void 0) { blocksValidate = false; }
    var code = maskCleaner(number);
    if (Number(code.substr(0, 1)) === 8) {
        if (code.length === 44)
            return barcodeAgreement(code);
        if (code.length === 48)
            return digitableLineAgreement(code, blocksValidate);
    }
    if (code.length === 44)
        return barcodeBank(code);
    if (code.length === 47)
        return digitableLineBank(code, blocksValidate);
    return false;
};

var getValueForBankType = function (number, codeType) {
    var value = '0';
    if (codeType === 'CODIGO DE BARRAS') {
        value = number.substr(9, 10);
    }
    if (codeType === 'LINHA DIGITAVEL') {
        value = number.substr(-8, 8);
    }
    return Number((parseInt(value, 10) / 100.0).toFixed(2));
};
var getValueForAgreementType = function (number, codeType) {
    var value = '0';
    if (codeType === 'CODIGO DE BARRAS') {
        value = number.substr(4, 11);
    }
    if (codeType === 'LINHA DIGITAVEL') {
        value = ("" + number.substr(0, 11) + number.substring(12)).substr(4, 11);
    }
    return Number((parseInt(value, 10) / 100.0).toFixed(2));
};
var getValue = function (number, type, codeType) {
    if (type === 'BANCO')
        return getValueForBankType(number, codeType);
    if (type === 'ARRECADACAO')
        return getValueForAgreementType(number, codeType);
    return 0;
};

var Boleto = /** @class */ (function () {
    /**
     * @param {string} number codigo de barras ou linha digitável de boletos bancários ou arrecadação
     */
    function Boleto(number) {
        this.number = maskCleaner(number);
        if (this.number.length === 46) {
            this.number = this.number + "0";
        }
    }
    /**
     * Identifica o tipo da numeração informada, se é código de barras, linha digitável ou inválida
     *
     * @return {string} CODIGO DE BARRAS
     * @return {string} LINHA DIGITAVEL
     * @return {string} INVALIDO
     */
    Boleto.prototype.codeType = function () {
        if (this.number.length === 44)
            return 'CODIGO DE BARRAS';
        if (this.number.length >= 46 && this.number.length <= 48)
            return 'LINHA DIGITAVEL';
        return 'INVALIDO';
    };
    /**
     * Identifica se o tipo do boleto é bancário ou arrecadação.
     * Em caso de arrecação é retornado também o subtipo
     *
     * @return {object} {BANCO, ''}
     * @return {object} {ARRECADACAO, ''}
     * @return {object} {ARRECADACAO, ARRECADACAO_PREFEITURA}
     * @return {object} {ARRECADACAO, CONVENIO_SANEAMENTO}
     * @return {object} {ARRECADACAO, CONVENIO_ENERGIA_ELETRICA_E_GAS}
     * @return {object} {ARRECADACAO, CONVENIO_TELECOMUNICACOES}
     * @return {object} {ARRECADACAO, ARRECADACAO_ORGAOS_GOVERNAMENTAIS}
     * @return {object} {ARRECADACAO, OUTROS}
     * @return {object} {ARRECADACAO, ARRECADACAO_TAXAS_DE_TRANSITO}
     */
    Boleto.prototype.type = function () {
        var type = this.number.substr(0, 2);
        if (Number(type[0]) === 8) {
            return typeMapping(Number(type));
        }
        return {
            type: 'BANCO',
            subtype: '',
        };
    };
    /**
     * Retorna a data de vencimento
     */
    Boleto.prototype.expirationDate = function () {
        var codeType = this.codeType();
        var typePaymentSlip = this.type();
        var febrabanDate = new Date('1997-10-07');
        var factor = null;
        if (codeType === 'LINHA DIGITAVEL') {
            var days = Number(this.number.substr(33, 4));
            factor = typePaymentSlip.type === 'BANCO'
                ? days * (24 * 60 * 60 * 1000)
                : 0;
        }
        if (codeType === 'CODIGO DE BARRAS') {
            var days = Number(this.number.substr(5, 4));
            factor = typePaymentSlip.type === 'BANCO'
                ? days * (24 * 60 * 60 * 1000)
                : 0;
        }
        return factor || factor === 0 ? new Date(febrabanDate.getTime() + factor) : febrabanDate;
    };
    /**
     * Verifica se o boleto esta vencido
     *
     * @return {boolean} true | false
     */
    Boleto.prototype.expired = function () {
        var type = this.type().type;
        if (type === 'ARRECADACAO')
            return false;
        var expirationDate = this.expirationDate();
        if (!expirationDate)
            return false;
        var _a = differenceForNow(expirationDate), days = _a[0], past = _a[1];
        return Boolean(days > 0 && past);
    };
    /**
     * Retorna a quantdade de dias vencidos
     */
    Boleto.prototype.expiredDays = function () {
        var type = this.type().type;
        if (type === 'ARRECADACAO')
            return 0;
        var expirationDate = this.expirationDate();
        var days = differenceForNow(expirationDate)[0];
        return this.expired() ? days : 0;
    };
    /**
     * Retorna o nome do banco
     */
    Boleto.prototype.banks = function () {
        var cod = this.number.substr(0, 3);
        return identifyBank(cod);
    };
    /**
     * Retorna o valor do boleto
     */
    Boleto.prototype.amount = function () {
        var type = this.type();
        var codeType = this.codeType();
        return getValue(this.number, type.type, codeType);
    };
    /**
     * Retorna o valor do boleto formatado (BRL)
     */
    Boleto.prototype.prettyAmount = function () {
        var value = this.amount();
        return currencyFormatter(value);
    };
    /**
     * Calcula e retorna o juros do boleto
     *
     * @param {interestValuenumber}  Porcentagem ou valor cobrado.
     * @param {number|boolean} expiredDays  Quantidade de dias vencidos, caso não informe a quantidade
     * ou informe false para este parâmetro, a quantidade de dias vencido é retirado do boleto.
     * @param {boolean} percent Forma de cobrança, por padrão é habilitado como porcentagem
     * caso seja em reais, atribua false
     * @param {boolean} month Informe se o juros é cobrado ao mês ou ao dia, para dia informe false
     *
     */
    Boleto.prototype.interest = function (interestValue, expiredDays, percent, month) {
        if (percent === void 0) { percent = true; }
        if (month === void 0) { month = true; }
        var valueBoleto = this.amount();
        var type = this.type().type;
        var expirationDate = this.expirationDate();
        var daysDiff = differenceForNow(expirationDate)[0];
        var days = typeof expiredDays !== 'number' && typeof daysDiff === 'number'
            ? daysDiff
            : expiredDays;
        return interstCalc(valueBoleto, days, interestValue, type, percent, month);
    };
    /**
     * Calcula e retorna a multa do boleto
     *
     * @param {number} finesValue Porcentagem ou valor cobrada.
     * @param {boolean} percent Forma de cobrança, por padrão é habilitado como porcentagem
     * caso seja em reais, atribua false
     */
    Boleto.prototype.fines = function (finesValue, percent) {
        if (percent === void 0) { percent = true; }
        var valueBoleto = this.amount();
        var expired = this.expired();
        var type = this.type().type;
        return finesCalc(valueBoleto, expired, type, finesValue, percent);
    };
    /**
     * Verifica se o boleto é valido
     *
     * @return {boolean} true | false
     */
    Boleto.prototype.valid = function () {
        return isValid(this.number);
    };
    /**
     * Retorna um objeto com todas informações do boleto.
     *
     * @return {object | null}
     */
    Boleto.prototype.toJSON = function () {
        var codeType = this.codeType();
        var type = this.type().type;
        if (codeType === 'INVALIDO' || !this.valid())
            return null;
        var bar = '';
        if (type === 'BANCO')
            bar = codeType === 'LINHA DIGITAVEL' ? toBarcodeBank(this.number) : this.number;
        if (type === 'ARRECADACAO')
            bar = codeType === 'LINHA DIGITAVEL' ? toBarcodeAgreement(this.number) : this.number;
        return {
            barcode: bar,
            codeType: codeType,
            type: type,
            expirationDate: this.expirationDate(),
            expired: this.expired(),
            expiredDays: this.expiredDays(),
            banks: this.banks(),
            amount: this.amount(),
            prettyAmount: this.prettyAmount(),
            valid: this.valid(),
        };
    };
    return Boleto;
}());

export { Boleto };
